/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */

import { CommonModule } from "@angular/common";
import { Component } from "@angular/core";
import { RouterModule } from "@angular/router";
import { ComicService } from "../comic-service.js";

@Component({
  imports:     [ CommonModule, RouterModule ],
  selector:    "app-archive",
  styleUrl:    "./archive.css",
  templateUrl: "./archive.html",
})
export class Archive {
  public comics: Array<{ number: number; title: string; date: string }> = [];
  public error:  string | undefined;
  public constructor(private readonly comicService: ComicService) {
    this.comicService.
      loadComics().
      then(() => {
        this.comics = this.comicService.getComics();
        this.error = undefined;
      }).
      catch((error: unknown) => {
        this.error = `Failed to load comics: ${
          error instanceof Error
            ? error.message
            : "Please check the browser console for more details."
        }`;
        console.error("Error loading comics:", error);
      });
  }
}
