/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */

import { Injectable } from "@angular/core";

const oneDay = 1000 * 60 * 60 * 24;

@Injectable({
  providedIn: "root",
})
export class ComicService {
  // eslint-disable-next-line @typescript-eslint/class-literal-property-style -- I should turn this rule off.
  private readonly comicsUrl: string = "https://cdn.yurigpt.com/comics.json";

  private ttl = 0;

  private comics: Array<{
    number: number;
    title:  string;
    date:   string;
  }> = [];

  public constructor() {
    void this.loadComics();
  }
  private static isObject(object: unknown): object is Record<string, unknown> {
    return typeof object === "object" && object !== null;
  }

  private static validateComics(
    comics: unknown,
  ): comics is Array<{ number: number; title: string; date: string }> {
    if (!Array.isArray(comics)) {
      return false;
    }

    return comics.every((comic) => {
      if (!ComicService.isObject(comic)) {
        return false;
      }
      return (
        "number" in comic
        && "title" in comic
        && "date" in comic
        && typeof comic["number"] === "number"
        && typeof comic["title"] === "string"
        && typeof comic["date"] === "string"
      );
    });
  }

  public async loadComics(): Promise<void> {
    if (this.ttl > Date.now()) {
      return;
    }
    this.comics = [];
    const response = await fetch(this.comicsUrl);
    const data: unknown = await response.json();
    if (ComicService.validateComics(data)) {
      this.comics = data.sort((a, b) => {
        return a.number - b.number;
      });
      this.ttl = Date.now() + oneDay;
    } else {
      console.error("Invalid comics data format:", data);
    }
  }

  public getComics(): Array<{ number: number; title: string; date: string }> {
    return this.comics;
  }

  public getComicById(
    id: string,
  ): { number: number; title: string; date: string } | undefined {
    const comicId = Number.parseInt(id, 10);
    return this.comics.find((comic) => {
      return comic.number === comicId;
    });
  }

  public getLatestComic():
    | { number: number; title: string; date: string }
    | undefined {
    return this.comics.at(-1);
  }

  public getLatestComicId(): string | undefined {
    const latestComic = this.getLatestComic();
    return latestComic
      ? latestComic.number.toString()
      : undefined;
  }
}
